@@ -36,7 +36,7 @@ toString(const RGBColor& color)
     {
       os.width(2);
       os.fill('0');
-      os << std::ios::hex << "#" << color.red << color.green << color.blue;
+      os << "#" << std::hex << color.red << color.green << color.blue;
     }
   return os.str();
 }
