@@ -21,7 +21,6 @@
 // <luca.padovani@cs.unibo.it>
 
 #include <config.h>
-#include <assert.h>
 #include <stdlib.h>
 
 #include <sstream>
@@ -95,14 +94,39 @@ enum GtkMathViewProperties
 
 /* structures */
 
-struct _GtkMathView {
+struct _GtkMathViewClass
+{
+  GtkWidgetClass parent_class;
+
+  void (*set_scroll_adjustments) (GtkMathView *math_view,
+				  GtkAdjustment *hadjustment,
+				  GtkAdjustment *vadjustment);
+
+  GtkMathViewModelSignal click;
+  GtkMathViewModelSignal select_begin;
+  GtkMathViewModelSignal select_over;
+  GtkMathViewModelSignal select_end;
+  GtkMathViewSelectAbortSignal select_abort;
+  GtkMathViewModelSignal element_over;
+
+  AbstractLogger* logger;
+  Configuration* configuration;
+  MathMLOperatorDictionary* dictionary;
+  Gtk_MathGraphicDevice* math_graphic_device;
+#if defined(ENABLE_BOXML)
+  Gtk_BoxGraphicDevice* box_graphic_device;
+#endif // ENABLE_BOXML
+};
+
+struct _GtkMathView
+{
   GtkWidget      parent;
 
   GtkWidget* 	 area;
   GdkPixmap*     pixmap;
 
-  guint 	 hsignal;
-  guint 	 vsignal;
+  guint 	 hsignal; // what is this for?
+  guint 	 vsignal; // what is this for?
 
   GtkAdjustment* hadjustment;
   GtkAdjustment* vadjustment;
@@ -127,29 +151,14 @@ struct _GtkMathView {
   GtkMathViewModelId cursor_elem;
   gint           cursor_index;
 
-  Configuration* configuration; // move to GtkMathViewClass?
-  MathMLOperatorDictionary* dictionary; // move to GtkMathViewClass?
   MathView*      view;
   Gtk_RenderingContext* renderingContext;
 };
 
-struct _GtkMathViewClass {
-  GtkWidgetClass parent_class;
-
-  void (*set_scroll_adjustments) (GtkMathView *math_view,
-				  GtkAdjustment *hadjustment,
-				  GtkAdjustment *vadjustment);
-
-  GtkMathViewModelSignal click;
-  GtkMathViewModelSignal select_begin;
-  GtkMathViewModelSignal select_over;
-  GtkMathViewModelSignal select_end;
-  GtkMathViewSelectAbortSignal select_abort;
-  GtkMathViewModelSignal element_over;
-};
-
 /* helper functions */
 
+static void gtk_math_view_base_class_init(GtkMathViewClass*);
+static void gtk_math_view_base_class_finalize(GtkMathViewClass*);
 static void gtk_math_view_class_init(GtkMathViewClass*);
 static void gtk_math_view_init(GtkMathView*);
 
@@ -209,6 +218,17 @@ findGtkWrapperArea(GtkMathView* math_view, GtkMathViewModelId node)
   return 0;
 }
 
+#if 0
+static GtkMathViewModelId
+findAction(GtkMathView* math_view, GtkMathViewModelId node)
+{
+  SmartPtr<Element> elem = math_view->view->elementOfModelElement(node);
+  while (elem && !is_a<MathMLActionElement>(elem) && !is_a<BoxMLActionElement>(elem))
+    elem = elem->getParent();
+  return elem ? math_view->view->modelElementOfElement(elem) : GtkMathViewModelId();
+}
+#endif
+
 /* auxiliary C functions */
 
 static void
@@ -339,7 +359,9 @@ static void
 gtk_math_view_paint(GtkMathView* math_view)
 {
   g_return_if_fail(math_view != NULL);
-  g_return_if_fail(math_view->configuration != 0);
+
+  GtkMathViewClass* math_view_class = GTK_MATH_VIEW_CLASS(G_OBJECT_GET_CLASS(G_OBJECT(math_view)));
+  g_return_if_fail(math_view_class != NULL);
 
   if (!GTK_WIDGET_MAPPED(GTK_WIDGET(math_view)) || math_view->freeze_counter > 0) return;
 
@@ -351,7 +373,7 @@ gtk_math_view_paint(GtkMathView* math_view)
   const gint height = widget->allocation.height;
 
   Gtk_RenderingContext* rc = math_view->renderingContext;
-  assert(rc);
+  g_return_if_fail(rc != 0);
 
   if (math_view->pixmap == NULL)
     {
@@ -360,11 +382,11 @@ gtk_math_view_paint(GtkMathView* math_view)
     }
 
   rc->setStyle(Gtk_RenderingContext::SELECTED_STYLE);
-  rc->setForegroundColor(math_view->configuration->getSelectForeground());
-  rc->setBackgroundColor(math_view->configuration->getSelectBackground());
+  rc->setForegroundColor(math_view_class->configuration->getSelectForeground());
+  rc->setBackgroundColor(math_view_class->configuration->getSelectBackground());
   rc->setStyle(Gtk_RenderingContext::NORMAL_STYLE);
-  rc->setForegroundColor(math_view->configuration->getForeground());
-  rc->setBackgroundColor(math_view->configuration->getBackground());
+  rc->setForegroundColor(math_view_class->configuration->getForeground());
+  rc->setBackgroundColor(math_view_class->configuration->getBackground());
 
   gdk_draw_rectangle(math_view->pixmap, widget->style->white_gc, TRUE, 0, 0, width, height);
 
@@ -420,8 +442,8 @@ GTKMATHVIEW_METHOD_NAME(get_type)(void)
       static const GTypeInfo math_view_info =
 	{
 	  sizeof(GtkMathViewClass),
-	  NULL,
-	  NULL,
+	  (GBaseInitFunc) gtk_math_view_base_class_init,
+	  (GBaseFinalizeFunc) gtk_math_view_base_class_finalize,
 	  (GClassInitFunc) gtk_math_view_class_init,
 	  NULL,
 	  NULL,
@@ -478,19 +500,89 @@ initOperatorDictionary(const SmartPtr<AbstractLogger>& logger, const SmartPtr<Co
 }
 
 static void
-gtk_math_view_class_init(GtkMathViewClass* klass)
+gtk_math_view_base_class_init(GtkMathViewClass* math_view_class)
 {
-  GObjectClass* gobject_class = G_OBJECT_CLASS(klass);
-  GtkObjectClass* object_class = (GtkObjectClass*) klass;
-  GtkWidgetClass* widget_class = (GtkWidgetClass*) klass;
+  g_return_if_fail(math_view_class != NULL);
+
+  SmartPtr<AbstractLogger> logger = Logger::create();
+  logger->ref();
+  math_view_class->logger = logger;
+
+  SmartPtr<Configuration> configuration = initConfiguration(logger, getenv("GTKMATHVIEWCONF"));
+  configuration->ref();
+  math_view_class->configuration = configuration;
+
+  SmartPtr<MathMLOperatorDictionary> dictionary = initOperatorDictionary(logger, configuration);
+  dictionary->ref();
+  math_view_class->dictionary = dictionary;
+
+  SmartPtr<Gtk_MathGraphicDevice> mathGraphicDevice = Gtk_MathGraphicDevice::create(logger, configuration);
+  mathGraphicDevice->ref();
+  math_view_class->math_graphic_device = mathGraphicDevice;
+
+#if defined(ENABLE_BOXML)
+  SmartPtr<Gtk_BoxGraphicDevice> boxGraphicDevice = Gtk_BoxGraphicDevice::create();
+  boxGraphicDevice->ref();
+  math_view_class->box_graphic_device = boxGraphicDevice;
+#endif // ENABLE_BOXML
+}
+
+static void
+gtk_math_view_base_class_finalize(GtkMathViewClass* math_view_class)
+{
+  g_return_if_fail(math_view_class != NULL);
+
+#if defined(ENABLE_BOXML)
+  if (math_view_class->box_graphic_device)
+    {
+      math_view_class->box_graphic_device->unref();
+      math_view_class->box_graphic_device = 0;
+    }
+#endif // ENABLE_BOXML
+
+  if (math_view_class->math_graphic_device)
+    {
+      math_view_class->math_graphic_device->unref();
+      math_view_class->math_graphic_device = 0;
+    }
+
+  if (math_view_class->dictionary)
+    {
+      math_view_class->dictionary->unref();
+      math_view_class->dictionary = 0;
+    }
+
+  if (math_view_class->configuration)
+    {
+      math_view_class->configuration->unref();
+      math_view_class->configuration = 0;
+    }
+
+  if (math_view_class->logger)
+    {
+      math_view_class->logger->unref();
+      math_view_class->logger = 0;
+    }
+}
+
+static void
+gtk_math_view_class_init(GtkMathViewClass* math_view_class)
+{
+  g_return_if_fail(math_view_class != NULL);
+
+#if 0
+  GObjectClass* gobject_class = G_OBJECT_CLASS(math_view_class);
+#endif
+  GtkObjectClass* object_class = (GtkObjectClass*) math_view_class;
+  GtkWidgetClass* widget_class = (GtkWidgetClass*) math_view_class;
 	
-  klass->click = gtk_math_view_click;
-  klass->select_begin = gtk_math_view_select_begin;
-  klass->select_over = gtk_math_view_select_over;
-  klass->select_end = gtk_math_view_select_end;
-  klass->select_abort = gtk_math_view_select_abort;
-  klass->element_over = gtk_math_view_element_over;
-  klass->set_scroll_adjustments = gtk_math_view_set_adjustments;
+  math_view_class->click = gtk_math_view_click;
+  math_view_class->select_begin = gtk_math_view_select_begin;
+  math_view_class->select_over = gtk_math_view_select_over;
+  math_view_class->select_end = gtk_math_view_select_end;
+  math_view_class->select_abort = gtk_math_view_select_abort;
+  math_view_class->element_over = gtk_math_view_element_over;
+  math_view_class->set_scroll_adjustments = gtk_math_view_set_adjustments;
 
   parent_class = (GtkWidgetClass*) gtk_type_class(gtk_widget_get_type());
 
@@ -670,8 +762,6 @@ gtk_math_view_init(GtkMathView* math_view)
   g_return_if_fail(math_view != NULL);
 
   math_view->pixmap          = NULL;
-  math_view->configuration   = 0;
-  math_view->dictionary      = 0;
   math_view->view            = 0;
   math_view->renderingContext = 0;
   math_view->freeze_counter  = 0;
@@ -683,33 +773,27 @@ gtk_math_view_init(GtkMathView* math_view)
   math_view->cursor_visible  = GTKMATHVIEW_CURSOR_OFF;
   math_view->hadjustment = NULL;
   math_view->vadjustment = NULL;
-
   math_view->top_x = math_view->top_y = 0;
   math_view->old_top_x = math_view->old_top_y = 0;
 
+#if 1
   SmartPtr<MathView> view = MathView::create();
   view->ref();
   math_view->view = view;
 
-  SmartPtr<AbstractLogger> logger = Logger::create();
-  view->setLogger(logger);
+  GtkMathViewClass* math_view_class = GTK_MATH_VIEW_CLASS(G_OBJECT_GET_CLASS(G_OBJECT(math_view)));
+  g_assert(math_view_class != NULL);
 
-  SmartPtr<Configuration> configuration = initConfiguration(logger, getenv("GTKMATHVIEWCONF"));
-  SmartPtr<MathMLOperatorDictionary> dictionary = initOperatorDictionary(logger, configuration);
-
-  configuration->ref();
-  math_view->configuration = configuration;
-
-  view->setOperatorDictionary(dictionary);
-  view->setMathMLNamespaceContext(MathMLNamespaceContext::create(view,
-								 Gtk_MathGraphicDevice::create(GTK_WIDGET(math_view))));
+  view->setLogger(math_view_class->logger);
+  view->setOperatorDictionary(math_view_class->dictionary);
+  view->setMathMLNamespaceContext(MathMLNamespaceContext::create(view, math_view_class->math_graphic_device));
 #if ENABLE_BOXML
-  view->setBoxMLNamespaceContext(BoxMLNamespaceContext::create(view,
-							       Gtk_BoxGraphicDevice::create(GTK_WIDGET(math_view))));
+  view->setBoxMLNamespaceContext(BoxMLNamespaceContext::create(view, math_view_class->box_graphic_device));
 #endif // ENABLE_BOXML
 
-  math_view->renderingContext = new Gtk_RenderingContext;
+  math_view->renderingContext = new Gtk_RenderingContext(math_view_class->logger);
   math_view->renderingContext->setColorMap(gtk_widget_get_colormap(GTK_WIDGET(math_view)));
+#endif
 }
 
 extern "C" GtkWidget*
@@ -732,12 +816,6 @@ gtk_math_view_destroy(GtkObject* object)
   math_view = GTK_MATH_VIEW(object);
   g_assert(math_view != NULL);
 
-  if (math_view->configuration)
-    {
-      math_view->configuration->unref();
-      math_view->configuration = 0;
-    }
-
   if (math_view->view)
     {
       math_view->view->resetRootElement();
@@ -843,8 +921,8 @@ gtk_math_view_size_request(GtkWidget* widget, GtkRequisition* requisition)
   g_return_if_fail(GTK_IS_MATH_VIEW(widget));
 
   GtkMathView* math_view = GTK_MATH_VIEW(widget);
-  g_assert(math_view);
-  g_assert(math_view->view);
+  g_return_if_fail(math_view != NULL);
+  g_return_if_fail(math_view->view != 0);
 
   if (BoundingBox box = math_view->view->getBoundingBox())
     {
@@ -1309,7 +1387,7 @@ GTKMATHVIEW_METHOD_NAME(unload)(GtkMathView* math_view)
 {
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(math_view->view != NULL);
-  math_view->view->resetRootElement();
+  math_view->view->unload();
   gtk_math_view_release_document_resources(math_view);
   gtk_math_view_paint(math_view);
 }
@@ -1330,6 +1408,7 @@ GTKMATHVIEW_METHOD_NAME(set_adjustments)(GtkMathView* math_view,
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(GTK_IS_MATH_VIEW(math_view));
 
+#if 0
   if (hadj != NULL)
     g_return_if_fail(GTK_IS_ADJUSTMENT(hadj));
   else
@@ -1339,43 +1418,52 @@ GTKMATHVIEW_METHOD_NAME(set_adjustments)(GtkMathView* math_view,
     g_return_if_fail(GTK_IS_ADJUSTMENT(vadj));
   else
     vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
-
-  if (math_view->hadjustment != NULL && (math_view->hadjustment != hadj))
-    {
-      gtk_signal_disconnect_by_data(GTK_OBJECT(math_view->hadjustment), math_view);
-      gtk_object_unref(GTK_OBJECT(math_view->hadjustment));
-    }
-
-  if (math_view->vadjustment != NULL && (math_view->vadjustment != vadj))
-    {
-      gtk_signal_disconnect_by_data(GTK_OBJECT(math_view->vadjustment), math_view);
-      gtk_object_unref(GTK_OBJECT(math_view->vadjustment));
-    }
+#endif
 
   if (math_view->hadjustment != hadj)
     {
+      if (math_view->hadjustment != NULL)
+	{
+	  gtk_signal_disconnect_by_data(GTK_OBJECT(math_view->hadjustment), math_view);
+	  gtk_object_unref(GTK_OBJECT(math_view->hadjustment));
+	}
+
       math_view->hadjustment = hadj;
-      gtk_object_ref(GTK_OBJECT(math_view->hadjustment));
-      gtk_object_sink(GTK_OBJECT(math_view->hadjustment));
-      
-      math_view->hsignal = 
-	g_signal_connect(GTK_OBJECT(hadj), 
-			 "value_changed",
-			 G_CALLBACK(hadjustment_value_changed),
-			 math_view);
+
+      if (hadj != NULL)
+	{
+	  gtk_object_ref(GTK_OBJECT(math_view->hadjustment));
+	  gtk_object_sink(GTK_OBJECT(math_view->hadjustment));
+	  
+	  math_view->hsignal = 
+	    g_signal_connect(GTK_OBJECT(hadj), 
+			     "value_changed",
+			     G_CALLBACK(hadjustment_value_changed),
+			     math_view);
+	}
     }
 
   if (math_view->vadjustment != vadj)
     {
+      if (math_view->vadjustment != NULL)
+	{
+	  gtk_signal_disconnect_by_data(GTK_OBJECT(math_view->vadjustment), math_view);
+	  gtk_object_unref(GTK_OBJECT(math_view->vadjustment));
+	}
+
       math_view->vadjustment = vadj;
-      gtk_object_ref(GTK_OBJECT(math_view->vadjustment));
-      gtk_object_sink(GTK_OBJECT(math_view->vadjustment));
-      
-      math_view->vsignal =
-	g_signal_connect(GTK_OBJECT(vadj), 
-			 "value_changed",
-			 G_CALLBACK(vadjustment_value_changed),
-			 math_view);
+
+      if (vadj != NULL)
+	{
+	  gtk_object_ref(GTK_OBJECT(math_view->vadjustment));
+	  gtk_object_sink(GTK_OBJECT(math_view->vadjustment));
+	  
+	  math_view->vsignal =
+	    g_signal_connect(GTK_OBJECT(vadj), 
+			     "value_changed",
+			     G_CALLBACK(vadjustment_value_changed),
+			     math_view);
+	}
     }
   
   setup_adjustments(math_view);
@@ -1418,56 +1506,74 @@ GTKMATHVIEW_METHOD_NAME(get_font_size)(GtkMathView* math_view)
   return math_view->view->getDefaultFontSize();
 }
 
-extern "C" void
+extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(structure_changed)(GtkMathView* math_view, GtkMathViewModelId elem)
 {
-  g_return_if_fail(math_view != NULL);
-  g_return_if_fail(math_view->view != NULL);
-  if (math_view->view->notifyStructureChanged(elem)) gtk_math_view_paint(math_view);
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
+  if (math_view->view->notifyStructureChanged(elem))
+    {
+      gtk_math_view_paint(math_view);
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
-extern "C" void
+extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(attribute_changed)(GtkMathView* math_view, GtkMathViewModelId elem, GtkMathViewModelString name)
 {
-  g_return_if_fail(math_view != NULL);
-  g_return_if_fail(math_view->view != NULL);
-  if (math_view->view->notifyAttributeChanged(elem, name)) gtk_math_view_paint(math_view);
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
+  if (math_view->view->notifyAttributeChanged(elem, name))
+    {
+      gtk_math_view_paint(math_view);
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
-extern "C" void
+extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(select)(GtkMathView* math_view, GtkMathViewModelId elem)
 {
-  g_return_if_fail(math_view);
-  g_return_if_fail(math_view->view);
-  g_return_if_fail(elem);
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
+  g_return_val_if_fail(elem != NULL, FALSE);
 
   if (SmartPtr<const Gtk_WrapperArea> area = findGtkWrapperArea(math_view, elem))
     {
       area->setSelected(1);
       gtk_math_view_paint(math_view);
+      return TRUE;
     }
+  else
+    return FALSE;
 }
 
-extern "C" void
+extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(unselect)(GtkMathView* math_view, GtkMathViewModelId elem)
 {
-  g_return_if_fail(math_view);
-  g_return_if_fail(math_view->view);
-  g_return_if_fail(elem);
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
+  g_return_val_if_fail(elem != NULL, FALSE);
 
   if (SmartPtr<const Gtk_WrapperArea> area = findGtkWrapperArea(math_view, elem))
     {
       area->setSelected(0);
       gtk_math_view_paint(math_view);
+      return TRUE;
     }
+  else
+    return FALSE;
 }
 
 extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(is_selected)(GtkMathView* math_view, GtkMathViewModelId elem)
 {
-  g_return_val_if_fail(math_view, FALSE);
-  g_return_val_if_fail(math_view->view, FALSE);
-  g_return_val_if_fail(elem, FALSE);
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
+  g_return_val_if_fail(elem != NULL, FALSE);
 
   if (SmartPtr<const Gtk_WrapperArea> area = findGtkWrapperArea(math_view, elem))
     return area->getSelected();
@@ -1749,6 +1855,7 @@ GTKMATHVIEW_METHOD_NAME(set_log_verbosity)(GtkMathView* math_view, gint level)
 {
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(math_view->view != 0);
+  g_return_if_fail(math_view->renderingContext != 0);
   math_view->view->getLogger()->setLogLevel(LogLevelId(level));
 }
 
@@ -1760,3 +1867,36 @@ GTKMATHVIEW_METHOD_NAME(get_log_verbosity)(GtkMathView* math_view)
   return math_view->view->getLogger()->getLogLevel();
 }
 
+extern "C" void
+GTKMATHVIEW_METHOD_NAME(set_t1_opaque_mode)(GtkMathView* math_view, gboolean mode)
+{
+  g_return_if_fail(math_view != NULL);
+  g_return_if_fail(math_view->renderingContext != 0);
+  math_view->renderingContext->setT1OpaqueMode(mode == TRUE);
+  gtk_math_view_paint(math_view);
+}
+
+extern "C" gboolean
+GTKMATHVIEW_METHOD_NAME(get_t1_opaque_mode)(GtkMathView* math_view)
+{
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->renderingContext != 0, FALSE);
+  return math_view->renderingContext->getT1OpaqueMode() ? TRUE : FALSE;
+}
+
+extern "C" void
+GTKMATHVIEW_METHOD_NAME(set_t1_anti_aliased_mode)(GtkMathView* math_view, gboolean mode)
+{
+  g_return_if_fail(math_view != NULL);
+  g_return_if_fail(math_view->renderingContext != 0);
+  math_view->renderingContext->setT1AntiAliasedMode(mode == TRUE);
+  gtk_math_view_paint(math_view);
+}
+
+extern "C" gboolean
+GTKMATHVIEW_METHOD_NAME(get_t1_anti_aliased_mode)(GtkMathView* math_view)
+{
+  g_return_val_if_fail(math_view != NULL, FALSE);
+  g_return_val_if_fail(math_view->renderingContext != 0, FALSE);
+  return math_view->renderingContext->getT1AntiAliasedMode() ? TRUE : FALSE;
+}
