@@ -60,6 +60,8 @@ static void file_close(GtkWidget*, gpointer);
 static void options_set_font_size(GtkWidget*, gpointer);
 static void options_change_font_size(GtkWidget*, gboolean);
 static void options_verbosity(GtkWidget*, guint);
+static void options_transparent(GtkWidget*, gpointer);
+static void options_anti_aliased(GtkWidget*, gpointer);
 static void options_selection(GtkWidget*, gboolean);
 static void selection_delete(GtkWidget*, gpointer);
 static void selection_parent(GtkWidget*, gpointer);
@@ -90,6 +92,9 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/Options/Verbosity/_Warnings",    NULL, options_verbosity,     1,  "/Options/Verbosity/Errors" },
   { "/Options/Verbosity/_Info",        NULL, options_verbosity,     2,  "/Options/Verbosity/Errors" },
   { "/Options/Verbosity/_Debug",       NULL, options_verbosity,     3,  "/Options/Verbosity/Errors" },
+  { "/Options/Type 1",                 NULL, NULL,                  0,  "<Branch>" },
+  { "/Options/Type 1/_Transparent",    NULL, options_transparent,   TRUE,  "<CheckItem>" },
+  { "/Options/Type 1/_Anti Aliased",   NULL, options_anti_aliased,  0,  "<CheckItem>" },
   { "/Options/sep1",                   NULL, NULL,                  0,  "<Separator>" },
   { "/Options/Selection/Structure",    NULL, options_selection,     0,  "<RadioItem>" },
   { "/Options/Selection/Semantics",    NULL, options_selection,     1,  "/Options/Selection/Structure" },
@@ -283,6 +288,22 @@ options_verbosity(GtkWidget* widget, guint level)
   gtk_math_view_set_log_verbosity(GTK_MATH_VIEW(main_area), level);
 }
 
+static void
+options_transparent(GtkWidget* widget, gpointer data)
+{
+  GtkMathView* math_view = GTK_MATH_VIEW(main_area);
+  g_return_if_fail(math_view != NULL);
+  gtk_math_view_set_t1_opaque_mode(math_view, !gtk_math_view_get_t1_opaque_mode(math_view));
+}
+
+static void
+options_anti_aliased(GtkWidget* widget, gpointer data)
+{
+  GtkMathView* math_view = GTK_MATH_VIEW(main_area);
+  g_return_if_fail(math_view != NULL);
+  gtk_math_view_set_t1_anti_aliased_mode(math_view, !gtk_math_view_get_t1_anti_aliased_mode(math_view));
+}
+
 static void
 options_selection(GtkWidget* widget, gboolean semantic)
 {
@@ -439,6 +460,7 @@ options_set_font_size(GtkWidget* widget, gpointer data)
 static void
 element_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
+  GdomeElement* action = NULL;
   GdomeDOMString* link = NULL;
 
   g_return_if_fail(math_view != NULL);
@@ -453,6 +475,15 @@ element_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 
   if (link != NULL)
     gdome_str_unref(link);
+
+  action = find_action_element(event->id);
+  if (action != NULL)
+    {
+      gtk_math_view_set_cursor(math_view, action, -1);
+      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_ON);
+    }
+  else
+    gtk_math_view_set_cursor(math_view, NULL, -1);
 }
 
 static void
@@ -538,8 +569,14 @@ select_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
       else
         root_selected = find_common_ancestor(first_selected, event->id);
 
-      if (root_selected != NULL)
-	gtk_math_view_select(math_view, root_selected);
+      while (root_selected != NULL && !gtk_math_view_select(math_view, root_selected))
+	{
+	  GdomeElement* new_root = gdome_el_parentNode(root_selected, &exc);
+	  g_assert(exc == 0);
+	  gdome_el_unref(root_selected, &exc);
+	  g_assert(exc == 0);
+	  root_selected = new_root;
+	}
 
       gtk_math_view_thaw(math_view);
     }
@@ -605,6 +642,7 @@ click(GtkMathView* math_view, const GtkMathViewModelEvent* event)
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(event != NULL);
 
+#if 0
   index = -1;
   if (gtk_math_view_get_char_at(math_view, event->x, event->y, &elem, &index, NULL, NULL))
     {
@@ -617,6 +655,7 @@ click(GtkMathView* math_view, const GtkMathViewModelEvent* event)
       /*printf("get_char_at: failed\n");*/
       gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_OFF);
     }
+#endif
 
   if (event->id != NULL)
     {
