@@ -35,14 +35,11 @@
 #include "defs.h"
 #include "guiGTK.h"
 #include "String.hh"
-#include "Globals.hh"
-#include "MathMLRenderingEngine.hh"
 #include "MathMLTokenElement.hh"
 #ifdef DEBUG
 #include "Gtk_GraphicsContext.hh"
 #endif // DEBUG
 #ifdef ENABLE_PROFILE
-#include "CharMapper.hh"
 #endif // ENABLE_PROFILE
 
 enum CommandLineOptionId {
@@ -61,7 +58,7 @@ extern void *parseMathMLFile(char *);
 
 static void printVersion()
 {
-  printf("%s - written by Luca Padovani (C) 2000-2001.\n", appName);
+  printf("%s - written by Luca Padovani (C) 2000-2004.\n", appName);
 #ifdef DEBUG
   //printf("Compiled %s %s\n", __DATE__, __TIME__);
 #endif // DEBUG
@@ -82,33 +79,13 @@ Usage: mathmlviewer [options] file ...\n\n\
   printf("%s\n", helpMsg);
 }
 
-#ifdef DEBUG
-static void
-checkCounters()
-{
-  Globals::dictionary.Unload();
-  Globals::logger(LOG_DEBUG, "Elements : %d", MathMLElement::GetCounter());
-  Globals::logger(LOG_DEBUG, "GCs      : %d", Gtk_GraphicsContext::GetCounter());
-  Globals::logger(LOG_DEBUG, "Strings  : %d", String::GetCounter());
-  Globals::logger(LOG_DEBUG, "Values   : %d (cached %d)", Value::GetCounter(), Value::GetCached());
-}
-#endif // DEBUG
-
-#ifdef ENABLE_PROFILE
-static void
-checkProfileCounters()
-{
-  Globals::logger(LOG_DEBUG, "Total : %d", CharMapper::GetChars());
-  Globals::logger(LOG_DEBUG, "alnum : %d", CharMapper::GetAlnumChars());
-}
-#endif // ENABLE_PROFILE
-
 int
 main(int argc, char *argv[])
 {
   bool debugMode = false;
+  gint logLevel = -1; 
 
-  sprintf(appName, "mathmlviewer v%s", "x.y.z");
+  sprintf(appName, "mathmlviewer v%s", VERSION);
 
   while (TRUE) {
     int option_index = 0;
@@ -145,8 +122,8 @@ main(int argc, char *argv[])
 
     case OPTION_VERBOSE:
     case 'v':
-      if (optarg == NULL) Globals::logger.SetLogLevel(0);
-      else Globals::logger.SetLogLevel(*optarg - '0');
+      if (optarg == NULL) logLevel = -1;
+      else logLevel = *optarg - '0';
       break;
 
     case '?':
@@ -162,12 +139,8 @@ main(int argc, char *argv[])
   atexit(checkCounters);
 #endif // DEBUG
 
-#ifdef ENABLE_PROFILE
-  atexit(checkProfileCounters);
-#endif // ENABLE_PROFILE
-
   if (optind < argc) {
-    GUI_init(&argc, &argv, appName, 500, 600);
+    GUI_init(&argc, &argv, appName, 500, 600, logLevel);
 
     while (optind < argc) {
 
@@ -184,7 +157,5 @@ main(int argc, char *argv[])
     GUI_uninit();
   } else printHelp();
 
-  Value::Flush();
-
   exit(0);
 }
